import collections
import os
import sys
import numpy as np

start_token = 'B'
end_token = 'E'


def process_poem(file_name):
    # 诗集
    poem = []
    with open(file_name, "r", encoding='utf-8', ) as f:
        for line in f.readlines():
            try:
                title, content = line.strip().split(':')
                content = content.replace(' ', '')
                if '_' in content or '(' in content or '（' in content or '《' in content or '[' in content or \
                        start_token in content or end_token in content:
                    continue
                if len(content) < 5 or len(content) > 79:
                    continue
                content = start_token + content + end_token
                poem.append(content)
            except ValueError as e:
                pass
    poem = sorted(poem, key=lambda l: len(line))

    all_words = []
    for poem in poem:
        all_words += [word for word in poem]
    counter = collections.Counter(all_words)
    count_pairs = sorted(counter.items(), key=lambda x: -x[1])
    words, _ = zip(*count_pairs)

    words = words[:len(words)] + (' ',)
    word_int_map = dict(zip(words, range(len(words))))
    poem_vector = [list(map(lambda word: word_int_map.get(word, len(words)), poem)) for poem in poem]

    return poem_vector, word_int_map, words


def generate_batch(batch_size, poem_vec, word_to_int):
    n_chunk = len(poem_vec) // batch_size
    x_batches = []
    y_batches = []
    for i in range(n_chunk):
        start_index = i * batch_size
        end_index = start_index + batch_size

        batches = poem_vec[start_index:end_index]
        length = max(map(len, batches))
        x_data = np.full((batch_size, length), word_to_int[' '], np.int32)
        for row in range(batch_size):
            x_data[row, :len(batches[row])] = batches[row]
        y_data = np.copy(x_data)
        y_data[:, :-1] = x_data[:, 1:]
        """
        x_data             y_data
        [6,2,4,6,9]       [2,4,6,9,9]
        [1,4,2,8,5]       [4,2,8,5,5]
        """
        x_batches.append(x_data)
        y_batches.append(y_data)
    return x_batches, y_batches
